/*
 * W1Device.cc
 *
 *  Created on: Oct 20, 2010
 *      Author: lamikr
 */
#include <iostream>
#include <fstream>
#include <sstream>
#include <iomanip>

#include <time.h>
#include <plp/log.h>

#include <plp/DeviceConfig.hh>
#include <plp/StoreDay.hh>

#include "Factory.hh"
#include "W1Device.hh"

using namespace std;
using namespace w1;
using namespace plp;

W1Device::W1Device(string device_id_param,
		string device_type_param,
		dirent *direntry_param) : SensorDevice(device_id_param, device_type_param) {
	string rootdir;
	string device_dir;
	string temp_str;

	type	= device_type_param;
	id	= device_id_param;
	if (direntry_param != NULL) {
		rootdir			= W1_SCAN_ROOTDIR;
		temp_str		= W1_SLAVE_FILE;
		dir_path		= rootdir + "/" + direntry_param->d_name;
		slave_file		= dir_path + "/" + temp_str;
		lifecycle_status	= LIFECYCLE_STATUS__AVAILABLE;
		log_debug("1-wire device's data file: %s\n", slave_file.c_str());
	}
	else {
		lifecycle_status	= LIFECYCLE_STATUS__UNAVAILABLE;
	}
	reader	= new DataReader(id);
	name	= "";
}

W1Device::~W1Device() {
	save_and_clean_cache();
	delete(reader);
}

string W1Device::get_name() {
	DeviceConfig	*cfg;

	if (name.empty() == true) {
		cfg	= DeviceConfig::get_device_config(id);
		if (cfg != NULL) {
			name	= cfg->get_cfg_value(DEVICE_CONFIG_VALUE_KEY__NAME);
			delete(cfg);
		}
	}
	return name;
}

void W1Device::set_name(string name_param) {
	DeviceConfig	*cfg;

	if (name.compare(name_param) != 0) {
		name	= name_param;
		cfg	= DeviceConfig::get_device_config(id);
		if (cfg != NULL) {
			cfg->set_cfg_value(DEVICE_CONFIG_VALUE_KEY__NAME, name_param);
			delete(cfg);
		}
	}
}

void W1Device::printout() {
	Data	*data;
	string	text;
	string	type;
	string  name;

	data	= get_data();
	if (data != NULL) {
		text	= data->to_string();
		cout << text << endl;
	}
	else {
		type	= get_type();
		name	= get_name();
		log_error("Could not read data for %s device: %s\n", type.c_str(), name.c_str());
	}
}

string W1Device::to_string(double dbl_val, int digit_count) {
	string		ret_val;
	ostringstream	out;

	out << fixed << setprecision(digit_count) << dbl_val;
	ret_val	= out.str();
	return ret_val;
}

Data *W1Device::get_data() {
	Data		*ret_val;
	vector<double>	*vect;

	log_debug("get_data() started\n");
	ret_val	= NULL;
	vect	= get_raw_data();
	if (vect != NULL) {
		log_debug("get_data() got raw data\n");
		ret_val	= new Data(vect, get_unit());
		cache(ret_val);
		delete(vect);
		log_debug("get_data() got raw data done\n");
	}
	else {
		log_debug("get_data() try to read old already saved data\n");
		// read old data already saved
		ret_val	= reader->get_latest_data();
		log_debug("get_data() try to read old already saved data done\n");
	}
	return ret_val;
}

void W1Device::cache(Data *data) {
	// TODO: add mutex for memory_cache
	memory_cache.push_back(data);
	if (memory_cache.size() > 5) {
		save_and_clean_cache();
	}
}

void W1Device::save_and_clean_cache() {
	list<Data *>::iterator	iter;
	Data			*data;
	int			dec_precision;

	dec_precision	= get_data_decimal_precision();
	StoreDay::save(id, &memory_cache, dec_precision);
	for(iter = memory_cache.begin(); iter != memory_cache.end(); iter++) {
		data	= (Data *)*iter;
		delete(data);
	}
	memory_cache.clear();
}

DataReader *W1Device::get_device_data() {
	return reader;
}
